import peewee as pw
from models import *


print '-'*80

# the dumb way
for article in Article.ALL:
  print 'title', article.title
  print 'author', article.author.first_name
  print 'editor', article.editor.first_name

  
print '-'*80

# a bit better
for article in Article.ALL.plus(Article.author):
  print 'title', article.title
  print 'author', article.author.first_name
  print 'editor', article.editor.first_name


print '-'*80

# much better
for article in Article.ALL.plus(Article.author).plus(Article.editor):
  print 'title', article.title
  print 'author', article.author.first_name
  print 'editor', article.editor.first_name


print '-'*80

# who edited all the articles i wrote?
q = Article.ALL \
  .plus(Article.author.as_('author')) \
  .plus(Article.editor) \
  .where(Person.as_('author').first_name=='Derek')
for article in q:
  print 'title', article.title
  print 'author', article.author.first_name
  print 'editor', article.editor.first_name



print '-'*80

# the peewee way
author_table = Person.alias()
editor_table = Person.alias()
q = Reply.select(Reply, author_table, editor_table) \
  .join(Article) \
  .join(author_table, join_type=pw.JOIN.LEFT_OUTER, on=(author_table==Article.author)) \
  .switch(Article) \
  .join(editor_table, join_type=pw.JOIN.LEFT_OUTER, on=(editor_table==Article.editor)) \
  .where(author_table.first_name=="Derek")
print q.sql()


# vs. the plus method
q = Reply.ALL \
  .plus(Reply.article, Article.author.as_('author')) \
  .plus(Reply.article, Article.editor) \
  .where(Person.as_('author').first_name=="Derek")
print q.sql()


print '-'*80

# grab everyone w/ all their articles
for person in Person.ALL.plus(Article.author):
  print person.first_name, person.article_set


print '-'*80

# grab everyone w/ all their articles and the article editor
for person in Person.ALL.plus(Article.author, Article.editor):
  articles = [{'article':article.title, 'editor':article.editor.first_name} for article in person.article_set]
  print person.first_name, articles


print '-'*80

# article with all their topics
for article in Article.ALL.plus(ArticleTopic.article, ArticleTopic.topic):
  print article.title, [at.topic.name for at in article.articletopic_set]






import peewee as pw
from models import *


# all the rows
for p in Person.ALL:
  print p, p.first_name


# basic condition
for p in Person.ALL.where(Person.first_name=='Derek'):
  print p, p.first_name


# get's the first object in a query
print Person.ALL.first().first_name


# get asserts only one result
print Person.ALL.get(Person.first_name=='Derek').first_name
try:
  print Person.ALL.get().first_name
except Exception as e:
  print e


# case-sensitive like (does not seem to workin sqlite)
print Person.ALL.where(Person.first_name % 'Der%').first()


# case-insensitive like
# sqlite's LIKE is by default case-insensitive,
# but docs say other dbs will use ILIKE
print Person.ALL.where(Person.first_name ** 'der%').get().first_name


# ordering
print [p.first_name for p in Person.ALL.order_by(Person.first_name)]






"""
Aggregate all the test modules and run from the command-line. For information
about running tests, see the README located in the `playhouse/tests` directory.
"""
import sys
import unittest

from playhouse.tests.test_apis import *
from playhouse.tests.test_compound_queries import *
from playhouse.tests.test_database import *
from playhouse.tests.test_fields import *
from playhouse.tests.test_helpers import *
from playhouse.tests.test_introspection import *
from playhouse.tests.test_keys import *
from playhouse.tests.test_models import *
from playhouse.tests.test_queries import *
from playhouse.tests.test_query_results import *
from playhouse.tests.test_transactions import *
from playhouse.tests.test_plus import *


if __name__ == '__main__':
    unittest.main(argv=sys.argv)



import os
import sys
import warnings
from distutils.core import setup
from distutils.extension import Extension
from distutils.version import StrictVersion

f = open(os.path.join(os.path.dirname(__file__), 'README.rst'))
readme = f.read()
f.close()

setup_kwargs = {}
try:
    from Cython.Distutils import build_ext
    from Cython import __version__ as cython_version
    if StrictVersion(cython_version) < StrictVersion('0.22.1'):
        warnings.warn('Your Cython appears to be an older version. You may '
                      'need to upgrade Cython in order to build the peewee '
                      'C extensions.')
except ImportError:
    cython_installed = False
else:
    cython_installed = True

speedups_ext_module = Extension(
    'playhouse._speedups',
    ['playhouse/_speedups.pyx'])
sqlite_udf_module = Extension(
    'playhouse._sqlite_udf',
    ['playhouse/_sqlite_udf.pyx'])
sqlite_ext_module = Extension(
    'playhouse._sqlite_ext',
    ['playhouse/_sqlite_ext.pyx'])


ext_modules = []
if cython_installed:
    ext_modules.extend([
        speedups_ext_module,
        sqlite_udf_module,
        sqlite_ext_module])

if ext_modules:
    setup_kwargs.update(
        cmdclass={'build_ext': build_ext},
        ext_modules=ext_modules)

setup(
    name='herman',
    version=__import__('peewee').__herman_version__,
    description='a little fork of peewee',
    long_description=readme,
    author='Derek Anderson',
    author_email='public@kered.org',
    url='http://github.com/keredson/peewee/',
    packages=['playhouse'],
    py_modules=['peewee', 'pwiz'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
    ],
    scripts = ['pwiz.py'],
    **setup_kwargs
)



import peewee as pw
from models import *


# create the instance
stefan = Person.create(first_name='Stefan')


# update the instance
stefan.first_name = "Steven" # much better
stefan.save()


# bulk update
Person.update(first_name='Derrick').where(Person.first_name=='Derek').execute()


# delete the instance
stefan.delete_instance()


# bulk delete
Person.delete().where(Person.first_name=='Derek').execute()


# burninate the countryside
Person.delete().execute()


# bump read count
Article.update(reads=Article.reads+1).where(Article.id==1).execute()


# bulk insert
Topic.insert_many([
  {'name': 'apple'},
  {'name': 'nyse'},
  {'name': 'tesla'},
]).execute()


